@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2017 David R. MacIver
+# Most of this work is copyright (C) 2013-2018 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -110,7 +110,6 @@ class settingsMeta(type):
 
 
 class settings(settingsMeta('settings', (object,), {})):
-
     """A settings object controls a variety of parameters that are used in
     falsification. These may control both the falsification strategy and the
     details of the data that is generated.
@@ -151,9 +150,7 @@ class settings(settingsMeta('settings', (object,), {})):
                     kwargs[setting.name] = getattr(defaults, setting.name)
                 else:
                     if kwargs[setting.name] != setting.future_default:
-                        if (
-                            setting.deprecation_message is not None
-                        ):
+                        if setting.deprecation_message is not None:
                             deprecations.append(setting)
                     if setting.validator:
                         kwargs[setting.name] = setting.validator(
@@ -292,11 +289,11 @@ class settings(settingsMeta('settings', (object,), {})):
 
     @staticmethod
     def register_profile(name, settings):
-        """registers a collection of values to be used as a settings profile.
-        These settings can be loaded in by name. Enable different defaults for
-        different settings.
+        """Registers a collection of values to be used as a settings profile.
 
-        - settings is a settings object
+        These settings can be loaded in by name. Enable different
+        defaults for different settings.  ``settings`` must be a
+        settings object.
 
         """
         settings._profiles[name] = settings
@@ -691,6 +688,40 @@ so.
 """
 )
 
+
+class PrintSettings(Enum):
+    """Flags to determine whether or not to print a detailed example blob to
+    use with :func:`~hypothesis.reproduce_failure` for failing test cases."""
+
+    NEVER = 0
+    """Never print a blob."""
+
+    INFER = 1
+    """Make an educated guess as to whether it would be appropriate to print
+    the blob.
+
+    The current rules are that this will print if both:
+
+    1. The output from Hypothesis appears to be unsuitable for use with
+       :func:`~hypothesis.example`.
+    2. The output is not too long."""
+
+    ALWAYS = 2
+    """Always print a blob on failure."""
+
+
+settings.define_setting(
+    'print_blob',
+    default=PrintSettings.INFER,
+    description="""
+Determines whether to print blobs after tests that can be used to reproduce
+failures.
+
+See :ref:`the documentation on @reproduce_failure <reproduce_failure>` for
+more details of this behaviour.
+"""
+)
+
 settings.lock_further_definitions()
 
 settings.register_profile('default', settings())
