@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2017 David R. MacIver
+# Most of this work is copyright (C) 2013-2018 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -31,6 +31,8 @@ from hypothesis.errors import InvalidArgument
 from hypothesis.control import reject
 from hypothesis.internal.compat import hrange
 from hypothesis.internal.coverage import check, check_function
+from hypothesis.internal.validation import check_type, try_convert, \
+    check_strategy, check_valid_size, check_valid_interval
 
 try:
     from pandas.api.types import is_categorical_dtype
@@ -63,7 +65,7 @@ def elements_and_dtype(elements, dtype, source=None):
         prefix = '%s.' % (source,)
 
     if elements is not None:
-        st.check_strategy(elements, '%selements' % (prefix,))
+        check_strategy(elements, '%selements' % (prefix,))
     else:
         with check('dtype is not None'):
             if dtype is None:
@@ -78,7 +80,7 @@ def elements_and_dtype(elements, dtype, source=None):
                     prefix,
                 ))
 
-    dtype = st.try_convert(np.dtype, dtype, 'dtype')
+    dtype = try_convert(np.dtype, dtype, 'dtype')
 
     if elements is None:
         elements = npst.from_dtype(dtype)
@@ -156,9 +158,11 @@ def range_indexes(min_size=0, max_size=None):
       it will default to some suitable value based on min_size.
 
     """
-    st.check_valid_interval(min_size, max_size, 'min_size', 'max_size')
+    check_valid_size(min_size, 'min_size')
+    check_valid_size(max_size, 'max_size')
     if max_size is None:
-        max_size = min_size + DEFAULT_MAX_SIZE
+        max_size = min([min_size + DEFAULT_MAX_SIZE, 2 ** 63 - 1])
+    check_valid_interval(min_size, max_size, 'min_size', 'max_size')
     return st.integers(min_size, max_size).map(pandas.RangeIndex)
 
 
@@ -187,8 +191,10 @@ def indexes(
       should be distinct.
 
     """
-    st.check_valid_interval(min_size, max_size, 'min_size', 'max_size')
-    st.check_type(bool, unique, 'unique')
+    check_valid_size(min_size, 'min_size')
+    check_valid_size(max_size, 'max_size')
+    check_valid_interval(min_size, max_size, 'min_size', 'max_size')
+    check_type(bool, unique, 'unique')
 
     elements, dtype = elements_and_dtype(elements, dtype)
 
@@ -237,7 +243,7 @@ def series(elements=None, dtype=None, index=None, fill=None, unique=False):
     if index is None:
         index = range_indexes()
     else:
-        st.check_strategy(index)
+        check_strategy(index)
 
     elements, dtype = elements_and_dtype(elements, dtype)
     index_strategy = index
@@ -427,11 +433,10 @@ def data_frames(
       InvalidArgument being raised.
 
     """
-
     if index is None:
         index = range_indexes()
     else:
-        st.check_strategy(index)
+        check_strategy(index)
 
     index_strategy = index
 
@@ -448,7 +453,7 @@ def data_frames(
                 @check_function
                 def row():
                     result = draw(rows)
-                    st.check_type(Iterable, result, 'draw(row)')
+                    check_type(Iterable, result, 'draw(row)')
                     return result
 
                 if len(index) > 0:
@@ -465,13 +470,13 @@ def data_frames(
             return rows_only()
 
     assert columns is not None
-    columns = st.try_convert(tuple, columns, 'columns')
+    columns = try_convert(tuple, columns, 'columns')
 
     rewritten_columns = []
     column_names = set()
 
     for i, c in enumerate(columns):
-        st.check_type(column, c, 'columns[%d]' % (i,))
+        check_type(column, c, 'columns[%d]' % (i,))
 
         c = copy(c)
         if c.name is None:
@@ -621,7 +626,7 @@ def data_frames(
                             seen.add(value)
                         if has_duplicate:
                             continue
-                    row = list(st.try_convert(tuple, row, 'draw(rows)'))
+                    row = list(try_convert(tuple, row, 'draw(rows)'))
 
                     if len(row) > len(rewritten_columns):
                         raise InvalidArgument((
